@@ -46,9 +46,9 @@ use crate::*;
 ///
 /// # Internal
 ///
-/// So far CompleteLayer will do two completion:
+/// So far `CompleteLayer` will do the following things:
 ///
-/// ## Read
+/// ## Read Completion
 ///
 /// OpenDAL requires all reader implements [`oio::Read`] and
 /// [`oio::BlockingRead`]. However, not all services have the
@@ -105,7 +105,7 @@ use crate::*;
 ///
 /// In this way, we can reduce the extra cost of dropping reader.
 ///
-/// ## List
+/// ## List Completion
 ///
 /// There are two styles of list, but not all services support both of
 /// them. CompleteLayer will add those capabilities in a zero cost way.
@@ -118,6 +118,12 @@ use crate::*;
 /// - if only `hierarchy`, with [`oio::to_hierarchy_pager`].
 /// - If neither not supported, something must be wrong.
 ///
+/// ## Capability Check
+///
+/// Before performing any operations, `CompleteLayer` will first check
+/// the operation against capability of the underlying service. If the
+/// operation is not supported, an error will be returned directly.
+///
 /// [`AccessorHint`]: crate::raw::AccessorHint
 pub struct CompleteLayer;
 
@@ -152,6 +158,10 @@ impl<A: Accessor> CompleteReaderAccessor<A> {
         args: OpRead,
     ) -> Result<(RpRead, CompleteReader<A, A::Reader>)> {
         let capability = self.meta.capability();
+        if !capability.read {
+            return new_capability_unsupported_error(Operation::Read);
+        }
+
         let seekable = capability.read_can_seek;
         let streamable = capability.read_can_next;
 
@@ -201,6 +211,10 @@ impl<A: Accessor> CompleteReaderAccessor<A> {
         args: OpRead,
     ) -> Result<(RpRead, CompleteReader<A, A::BlockingReader>)> {
         let capability = self.meta.capability();
+        if !capability.read || !capability.blocking {
+            return new_capability_unsupported_error(Operation::BlockingRead);
+        }
+
         let seekable = capability.read_can_seek;
         let streamable = capability.read_can_next;
 
@@ -343,6 +357,11 @@ impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
     }
 
     async fn stat(&self, path: &str, args: OpStat) -> Result<RpStat> {
+        let capability = self.meta.capability();
+        if !capability.stat {
+            return new_capability_unsupported_error(Operation::Stat);
+        }
+
         self.inner.stat(path, args).await.map(|v| {
             v.map_metadata(|m| {
                 let bit = m.bit();
@@ -352,6 +371,11 @@ impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
     }
 
     fn blocking_stat(&self, path: &str, args: OpStat) -> Result<RpStat> {
+        let capability = self.meta.capability();
+        if !capability.stat || !capability.blocking {
+            return new_capability_unsupported_error(Operation::BlockingStat);
+        }
+
         self.inner.blocking_stat(path, args).map(|v| {
             v.map_metadata(|m| {
                 let bit = m.bit();
@@ -361,6 +385,11 @@ impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
     }
 
     async fn write(&self, path: &str, args: OpWrite) -> Result<(RpWrite, Self::Writer)> {
+        let capability = self.meta.capability();
+        if !capability.write {
+            return new_capability_unsupported_error(Operation::Write);
+        }
+
         let size = args.content_length();
         self.inner
             .write(path, args)
@@ -369,6 +398,11 @@ impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
     }
 
     fn blocking_write(&self, path: &str, args: OpWrite) -> Result<(RpWrite, Self::BlockingWriter)> {
+        let capability = self.meta.capability();
+        if !capability.write || !capability.blocking {
+            return new_capability_unsupported_error(Operation::BlockingWrite);
+        }
+
         let size = args.content_length();
         self.inner
             .blocking_write(path, args)
@@ -376,19 +410,124 @@ impl<A: Accessor> LayeredAccessor for CompleteReaderAccessor<A> {
     }
 
     async fn append(&self, path: &str, args: OpAppend) -> Result<(RpAppend, Self::Appender)> {
+        let capability = self.meta.capability();
+        if !capability.append {
+            return new_capability_unsupported_error(Operation::Append);
+        }
+
         self.inner
             .append(path, args)
             .await
             .map(|(rp, a)| (rp, CompleteAppender::new(a)))
     }
 
+    async fn create_dir(&self, path: &str, args: OpCreateDir) -> Result<RpCreateDir> {
+        let capability = self.meta.capability();
+        if !capability.create_dir {
+            return new_capability_unsupported_error(Operation::CreateDir);
+        }
+
+        self.inner().create_dir(path, args).await
+    }
+
+    fn blocking_create_dir(&self, path: &str, args: OpCreateDir) -> Result<RpCreateDir> {
+        let capability = self.meta.capability();
+        if !capability.create_dir || !capability.blocking {
+            return new_capability_unsupported_error(Operation::BlockingCreateDir);
+        }
+
+        self.inner().blocking_create_dir(path, args)
+    }
+
+    async fn delete(&self, path: &str, args: OpDelete) -> Result<RpDelete> {
+        let capability = self.meta.capability();
+        if !capability.delete {
+            return new_capability_unsupported_error(Operation::Delete);
+        }
+
+        self.inner().delete(path, args).await
+    }
+
+    fn blocking_delete(&self, path: &str, args: OpDelete) -> Result<RpDelete> {
+        let capability = self.meta.capability();
+        if !capability.delete || !capability.blocking {
+            return new_capability_unsupported_error(Operation::BlockingDelete);
+        }
+
+        self.inner().blocking_delete(path, args)
+    }
+
+    async fn copy(&self, from: &str, to: &str, args: OpCopy) -> Result<RpCopy> {
+        let capability = self.meta.capability();
+        if !capability.copy {
+            return new_capability_unsupported_error(Operation::Copy);
+        }
+
+        self.inner().copy(from, to, args).await
+    }
+
+    fn blocking_copy(&self, from: &str, to: &str, args: OpCopy) -> Result<RpCopy> {
+        let capability = self.meta.capability();
+        if !capability.copy || !capability.blocking {
+            return new_capability_unsupported_error(Operation::BlockingCopy);
+        }
+
+        self.inner().blocking_copy(from, to, args)
+    }
+
+    async fn rename(&self, from: &str, to: &str, args: OpRename) -> Result<RpRename> {
+        let capability = self.meta.capability();
+        if !capability.rename {
+            return new_capability_unsupported_error(Operation::Rename);
+        }
+
+        self.inner().rename(from, to, args).await
+    }
+
+    fn blocking_rename(&self, from: &str, to: &str, args: OpRename) -> Result<RpRename> {
+        let capability = self.meta.capability();
+        if !capability.rename || !capability.blocking {
+            return new_capability_unsupported_error(Operation::BlockingRename);
+        }
+
+        self.inner().blocking_rename(from, to, args)
+    }
+
     async fn list(&self, path: &str, args: OpList) -> Result<(RpList, Self::Pager)> {
+        let capability = self.meta.capability();
+        if !capability.list {
+            return new_capability_unsupported_error(Operation::List);
+        }
+
         self.complete_list(path, args).await
     }
 
     fn blocking_list(&self, path: &str, args: OpList) -> Result<(RpList, Self::BlockingPager)> {
+        let capability = self.meta.capability();
+        if !capability.list || !capability.blocking {
+            return new_capability_unsupported_error(Operation::BlockingList);
+        }
+
         self.complete_blocking_list(path, args)
     }
+
+    async fn presign(&self, path: &str, args: OpPresign) -> Result<RpPresign> {
+        let capability = self.meta.capability();
+        if !capability.presign {
+            return new_capability_unsupported_error(Operation::Presign);
+        }
+
+        self.inner.presign(path, args).await
+    }
+
+    async fn batch(&self, args: OpBatch) -> Result<RpBatch> {
+        let capability = self.meta.capability();
+        if !capability.batch {
+            return new_capability_unsupported_error(Operation::Batch);
+        }
+
+        self.inner().batch(args).await
+    }
 }
 
 pub enum CompleteReader<A: Accessor, R> {
@@ -568,6 +707,27 @@ where
         Ok(())
     }
 
+    async fn sink(&mut self, size: u64, s: oio::Streamer) -> Result<()> {
+        if let Some(total_size) = self.size {
+            if self.written + size > total_size {
+                return Err(Error::new(
+                    ErrorKind::ContentTruncated,
+                    &format!(
+                        "writer got too much data, expect: {size}, actual: {}",
+                        self.written + size
+                    ),
+                ));
+            }
+        }
+
+        let w = self.inner.as_mut().ok_or_else(|| {
+            Error::new(ErrorKind::Unexpected, "writer has been closed or aborted")
+        })?;
+        w.sink(size, s).await?;
+        self.written += size;
+        Ok(())
+    }
+
     async fn abort(&mut self) -> Result<()> {
         let w = self.inner.as_mut().ok_or_else(|| {
             Error::new(ErrorKind::Unexpected, "writer has been closed or aborted")
@@ -701,3 +861,161 @@ where
         Ok(())
     }
 }
+
+fn new_capability_unsupported_error<R>(operation: Operation) -> Result<R> {
+    Err(Error::new(ErrorKind::Unsupported, "operation is not supported").with_operation(operation))
+}
+
+#[cfg(test)]
+mod tests {
+    use std::collections::HashMap;
+    use std::time::Duration;
+
+    use async_trait::async_trait;
+    use http::HeaderMap;
+    use http::Method as HttpMethod;
+
+    use super::*;
+
+    #[derive(Default)]
+    struct MockBuilder {
+        capability: Capability,
+    }
+
+    impl MockBuilder {
+        fn with_capacity(mut self, capability: Capability) -> Self {
+            self.capability = capability;
+            self
+        }
+    }
+
+    impl Builder for MockBuilder {
+        const SCHEME: Scheme = Scheme::Custom("mock");
+        type Accessor = MockService;
+
+        fn from_map(_: HashMap<String, String>) -> Self {
+            Self::default()
+        }
+
+        fn build(&mut self) -> Result<Self::Accessor> {
+            Ok(MockService {
+                capability: self.capability,
+            })
+        }
+    }
+
+    #[derive(Debug)]
+    struct MockService {
+        capability: Capability,
+    }
+
+    #[async_trait]
+    impl Accessor for MockService {
+        type Reader = ();
+        type BlockingReader = ();
+        type Writer = ();
+        type BlockingWriter = ();
+        type Appender = ();
+        type Pager = ();
+        type BlockingPager = ();
+
+        fn info(&self) -> AccessorInfo {
+            let mut info = AccessorInfo::default();
+            info.set_capability(self.capability);
+
+            info
+        }
+
+        async fn stat(&self, _: &str, _: OpStat) -> Result<RpStat> {
+            Ok(RpStat::new(Metadata::new(EntryMode::Unknown)))
+        }
+
+        async fn read(&self, _: &str, _: OpRead) -> Result<(RpRead, Self::Reader)> {
+            Ok((RpRead::new(0), ()))
+        }
+
+        async fn write(&self, _: &str, _: OpWrite) -> Result<(RpWrite, Self::Writer)> {
+            Ok((RpWrite::new(), ()))
+        }
+
+        async fn append(&self, _: &str, _: OpAppend) -> Result<(RpAppend, Self::Appender)> {
+            Ok((RpAppend::new(), ()))
+        }
+
+        async fn copy(&self, _: &str, _: &str, _: OpCopy) -> Result<RpCopy> {
+            Ok(RpCopy {})
+        }
+
+        async fn create_dir(&self, _: &str, _: OpCreateDir) -> Result<RpCreateDir> {
+            Ok(RpCreateDir {})
+        }
+
+        async fn delete(&self, _: &str, _: OpDelete) -> Result<RpDelete> {
+            Ok(RpDelete {})
+        }
+
+        async fn list(&self, _: &str, _: OpList) -> Result<(RpList, Self::Pager)> {
+            Ok((RpList {}, ()))
+        }
+
+        async fn rename(&self, _: &str, _: &str, _: OpRename) -> Result<RpRename> {
+            Ok(RpRename {})
+        }
+
+        async fn presign(&self, _: &str, _: OpPresign) -> Result<RpPresign> {
+            Ok(RpPresign::new(PresignedRequest::new(
+                HttpMethod::POST,
+                "https://example.com/presign".parse().expect("should parse"),
+                HeaderMap::new(),
+            )))
+        }
+    }
+
+    /// Perform the test against different capability preconditions.
+    macro_rules! capability_test {
+        ($cap:ident, |$arg:ident| { $($body:tt)* }) => {
+            paste::item! {
+                #[tokio::test]
+                async fn [<test_capability_ $cap>]() {
+                    let res_builder = |$arg: Operator| async move {
+                        let res = { $($body)* };
+                        res.await.err()
+                    };
+
+                    let builder = MockBuilder::default().with_capacity(Capability {
+                        $cap: false,
+                        ..Default::default()
+                    });
+                    let op = Operator::new(builder).expect("should build").finish();
+                    let res = res_builder(op.clone()).await;
+                    assert_eq!(res.expect("should not be None").kind(), ErrorKind::Unsupported);
+
+                    let builder = MockBuilder::default().with_capacity(Capability {
+                        $cap: true,
+                        ..Default::default()
+                    });
+                    let op = Operator::new(builder).expect("should build").finish();
+                    let res = res_builder(op.clone()).await;
+                    assert!(res.is_none());
+                }
+            }
+        };
+    }
+
+    capability_test!(stat, |op| { op.stat("/path/to/mock_file") });
+    capability_test!(read, |op| { op.read("/path/to/mock_file") });
+    capability_test!(write, |op| { op.writer("/path/to/mock_file") });
+    capability_test!(append, |op| { op.appender("/path/to/mock_file") });
+    capability_test!(create_dir, |op| { op.create_dir("/path/to/mock_dir/") });
+    capability_test!(delete, |op| { op.delete("/path/to/mock_file") });
+    capability_test!(copy, |op| {
+        op.copy("/path/to/mock_file", "/path/to/mock_file_2")
+    });
+    capability_test!(rename, |op| {
+        op.rename("/path/to/mock_file", "/path/to/mock_file_2")
+    });
+    capability_test!(list, |op| { op.list("/path/to/mock_dir/") });
+    capability_test!(presign, |op| {
+        op.presign_read("/path/to/mock_file", Duration::from_secs(1))
+    });
+}
