@@ -88,7 +88,7 @@ pub async fn test_list_dir(op: Operator) -> Result<()> {
 pub async fn test_list_rich_dir(op: Operator) -> Result<()> {
     op.create_dir("test_list_rich_dir/").await?;
 
-    let mut expected: Vec<String> = (0..=1000)
+    let mut expected: Vec<String> = (0..=100)
         .map(|num| format!("test_list_rich_dir/file-{num}"))
         .collect();
 
@@ -105,7 +105,7 @@ pub async fn test_list_rich_dir(op: Operator) -> Result<()> {
         .collect::<Vec<_>>()
         .await;
 
-    let mut objects = op.list("test_list_rich_dir/").await?;
+    let mut objects = op.with_limit(10).list("test_list_rich_dir/").await?;
     let mut actual = vec![];
     while let Some(o) = objects.try_next().await? {
         let path = o.path().to_string();
